//+------------------------------------------------------------------+
//|                                  SpielersHedge Divergence_v2.mq4 |
//|                                             2008.07.12 SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " 2008.07.12 SwingMan"
#property link ""
//+------------------------------------------------------------------+
//  v2 - write infos, like DivergenceStdDev v_1
//+------------------------------------------------------------------+

#property indicator_separate_window

#include <SpielersHedge_Library.mqh>

#property indicator_buffers 6
//---- colors
#property indicator_color1 DeepPink    // highs
#property indicator_color2 LimeGreen    // lows
#property indicator_color3 Tomato
#property indicator_color4 Tomato
#property indicator_color5 DeepSkyBlue
#property indicator_color6 DeepSkyBlue
   
//---- Levels
#property indicator_level1 0
#property indicator_levelstyle STYLE_SOLID
#property indicator_levelcolor Gold


//---- input parameters --------------------------------------------
extern double FirstDivergenceLimit = 0.20;
extern double DivergenceStep = 0.20;
//------------------------------------------------------------------


//---- constants
string sWindow1 = "SpielersHedge Divergence_v2";
string sObject = "SHDiv_2_";

//---- Buffers
double RatioHighs[], RatioLows[];
double levelUp1[], levelUp2[], levelDn1[], levelDn2[];

//---- Arrays

//---- variables
string sWindowName;
double lastClose3, CurrentRange;
int iWindow, Trade_Direction;

//-- constants
int fontSize_Text = 8;
int iLabelCorner = 1;
int xx1 = 80;
int xx2 = 2;
int xx3 = 60;
int xx4 = 30;
int yy0 = 5, yy;
int yStep = 10;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{   
   string st = DoubleToStr(FirstDivergenceLimit,2) + ", " + DoubleToStr(DivergenceStep,2);
   sWindowName = sWindow1 + " (" + st +") ";
   IndicatorShortName(sWindowName);
     
//---- Moving Averages ----------------------------------  
   SetIndexBuffer(0, RatioHighs);
   SetIndexBuffer(1, RatioLows);
   SetIndexBuffer(2, levelUp1);
   SetIndexBuffer(3, levelUp2);
   SetIndexBuffer(4, levelDn1);
   SetIndexBuffer(5, levelDn2);
   
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,1);
   SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,1);
   SetIndexStyle(2,DRAW_LINE,STYLE_SOLID,1);
   SetIndexStyle(3,DRAW_LINE,STYLE_SOLID,1);
   SetIndexStyle(4,DRAW_LINE,STYLE_SOLID,1);
   SetIndexStyle(5,DRAW_LINE,STYLE_SOLID,1);
   
   SetIndexLabel(0,"Ratio SHORT trade");
   SetIndexLabel(1,"Ratio LONG trade");
   SetIndexLabel(2,NULL);
   SetIndexLabel(3,NULL);
   SetIndexLabel(4,NULL);
   SetIndexLabel(5,NULL);
   
//---- initialization done   
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{ return(0); }

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int i, y, limit;  
   datetime TimeArray[];   
   ArrayCopySeries(TimeArray,MODE_TIME,Symbol(),PERIOD_D1); 
   
   int counted_bars=IndicatorCounted();  
   if (counted_bars>0) counted_bars--;  
   limit=Bars - counted_bars;
        
//---- 1. Calculation ------------------------------------------------
   for(i=0,y=0;i<limit;i++)
   {
      if (Time[i]<TimeArray[y]) y++; 
      
      lastClose3 = iClose(Symbol(), PERIOD_D1, y+1);
      if (i==0) double CurrentLastClose3 = lastClose3;
      
      double dHigh = iHigh(Symbol(), Period(), i);
      double dLow  = iLow(Symbol(), Period(), i);
      if (lastClose3 != 0) {
         RatioHighs[i] = 100*(dHigh - lastClose3) / lastClose3;
         RatioLows[i]  = 100*(dLow - lastClose3) / lastClose3;
         
         double firstDifference =  FirstDivergenceLimit;
         double stepDifference  =  DivergenceStep;
   
         double lineUp1 =  firstDifference;
         double lineDn1 = -firstDifference;

         double lineUp2 = lineUp1 + stepDifference;
         double lineDn2 = lineDn1 - stepDifference;         
         
         levelUp1[i] = lineUp1;
         levelUp2[i] = lineUp2;
         levelDn1[i] = lineDn1;
         levelDn2[i] = lineDn2;
      }
   }
   
   double dClose = iClose(Symbol(), Period(), 0);
   CurrentRange = 100*(dClose - CurrentLastClose3) / CurrentLastClose3;
   
//-- write results   
   Write_TradingLevels();
   Write_CurrentRange();   
//----
   return(0);
}
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//    Write Trading Levels
//+------------------------------------------------------------------+
void Write_TradingLevels()
{
   string sLevel1, sLevel2, sLevel3; 
   iWindow = WindowFind(sWindowName);
   int iBar = 0, iLevel;
     
   yy = yy0;
   iLabelCorner = 1;
   color dColor = Orange;
   color dColorArrow;
   int iHeight = 10;

   //-- get trade direction
   double dRatioHighs = RatioHighs[iBar];
   double dRatioLows  = RatioLows[iBar];
   
   double Diff_LongTrade  = MathAbs(dRatioLows  - levelDn1[iBar]);
   double Diff_ShortTrade = MathAbs(dRatioHighs - levelUp1[iBar]);
   
   if (CurrentRange < 0) Trade_Direction = 1; else Trade_Direction = -1;
   
   //-- Trade LONG ...................................................
   if (Trade_Direction == 1) {
      dColorArrow = Green;
      if (CurrentRange < levelDn1[iBar]) {
         string sTrend = "LONG trade";
         int ArrowCode = 233;
      }  else {        
         sTrend = "possible long trade at " + DoubleToStr(levelDn1[iBar],2)+ "%";
         ArrowCode = 216;
      }
      //-- levels, CurrentRange < 0
      if (CurrentRange < levelDn1[iBar]) iLevel = 0;
      if (CurrentRange <= levelDn1[iBar] && CurrentRange > levelDn2[iBar]) iLevel = 1;
      if (CurrentRange <= levelDn2[iBar]) iLevel = 2;
      //if (CurrentRange <= levelDn2[iBar] && CurrentRange > levelDn3[iBar]) iLevel = 2;
      //if (CurrentRange <= levelDn3[iBar]) iLevel = 3;
      sLevel1 = "level 1 LONG trade= "  + DoubleToStr(levelDn1[iBar],2)+ "%";
      sLevel2 = "level 2 LONG trade= "  + DoubleToStr(levelDn2[iBar],2)+ "%";
      //sLevel3 = "level 3 LONG trade= "  + DoubleToStr(levelDn3[iBar],2)+ "%";
   } else   
   
   //-- Trade SHORT ..................................................
   {
      dColorArrow = Red;
      if (CurrentRange >= levelUp1[iBar]) {
         sTrend = "SHORT trade";
         ArrowCode = 234;
      } else {
         sTrend = "possible short trade at " + DoubleToStr(levelUp1[iBar],2)+ "%";
         ArrowCode = 216;
      }
      //-- levels, CurrentRange > 0
      if (CurrentRange > levelUp1[iBar]) iLevel = 0;
      if (CurrentRange >= levelUp1[iBar] && CurrentRange < levelUp2[iBar]) iLevel = 1;
      if (CurrentRange >= levelUp2[iBar]) iLevel = 2;
      //if (CurrentRange >= levelUp2[iBar] && CurrentRange < levelUp3[iBar]) iLevel = 2;
      //if (CurrentRange >= levelUp3[iBar]) iLevel = 3;
      sLevel1 = "level 1 SHORT trade= "  + DoubleToStr(levelUp1[iBar],2)+ "%";
      sLevel2 = "level 2 SHORT trade= "  + DoubleToStr(levelUp2[iBar],2)+ "%";
      //sLevel3 = "level 3 SHORT trade= "  + DoubleToStr(levelUp3[iBar],2)+ "%";
   }
   
   //-- write text --------------------------------------------------
   //-- range and level
   string sRange = "Range: " + DoubleToStr(CurrentRange,2) + "%";
   string sLevel = "Level: " + iLevel;
   Write_Label(iWindow, iLabelCorner, sObject, sRange, "", fontSize_Text, dColor, xx2, xx2, yy); yy = yy + yStep;
   Write_Label(iWindow, iLabelCorner, sObject, sLevel, "", fontSize_Text, dColor, xx2, xx2, yy); yy = yy + yStep;
   //-- trading levels
   yy = yy + 5;
   Write_Label(iWindow, iLabelCorner, sObject, sLevel1, "", fontSize_Text, dColor, xx2, xx2, yy); yy = yy + yStep;
   Write_Label(iWindow, iLabelCorner, sObject, sLevel2, "", fontSize_Text, dColor, xx2, xx2, yy); yy = yy + yStep;
   Write_Label(iWindow, iLabelCorner, sObject, sLevel3, "", fontSize_Text, dColor, xx2, xx2, yy); yy = yy + yStep;
  
   //-- trend
   dColor = Gold;
   yy = yy + 5;   
   Write_Label(iWindow, iLabelCorner, sObject, sTrend, "", fontSize_Text, dColor, xx2+25, xx2, yy);
   SetArrowObject(iWindow, iLabelCorner, sObject + "a", ArrowCode, 
                    iHeight, dColorArrow, xx2 + 7, yy);
}

//+------------------------------------------------------------------+
//    Write Current Range
//+------------------------------------------------------------------+
void Write_CurrentRange()
{
   //-- write CurrentRange
   int iTextCorner = 0;
   double fontSize = 9;
   double fontFactor = 1.0;
   if (CurrentRange > 0) 
      color TextColor = Red ; else    // short trade
            TextColor = DeepSkyBlue;         // long trade
   datetime xx = Time[0];
   //-- y position
   if (CurrentRange>0) {
      //fontFactor = 0.5;
      string sign = "+";
   } else {
      //fontFactor = 1.5;
      sign = "";
   }   
   
   double yy = CurrentRange + (fontSize*fontFactor) / 100.0;
   string text = "                " + sign + DoubleToStr(CurrentRange,2)+ "%";
   Write_Text(sObject+"Range", text, iWindow, iTextCorner, xx, yy, fontSize, TextColor);
}   